function [contig_all, overlap_all, name_hash_all, deltafiles_all] = align_contigs_sets(chromosome_fasta, contigs_sets_fasta, contig_set_names, quality, clipping_thrs, z_weights, distance,max_chromosome_dist)
    % Driver to align the mulitple fasta files 'contigs_sets_fasta' against the fasta file 'chromosome_fasta' 
        %
        % INPUT
        % chromosome_fasta      Fasta file location of reference chomosome
        % contigs_sets_fasta    Cell array with multiple fasta files with assemblies
        % contig_set_names      Cell array with names of the assembly
        % quality               Quality of the assemblies
        % clipping_thrs         How much non-aligned overlap to allow (in number of nucleotides)
        %
        % OUPUT
        % contig_all            Structarray with contig properties 
        % overlap_all           Structarray with overlap properties
        % name_hash_all         Name hash wit key: contig_name, value: position in contig array
    
        contig_all     = [];
        overlap_all    = [];
        deltafiles_all = [];
        name_hash_all  = containers.Map;
        
        for ic = 1:length(contigs_sets_fasta)
            
            % Get data for contig_set number 'i'
            [adj_new, names_new, name_hash_new, contig_new, overlap_new, deltafile_new] = ...
                align_contigs(chromosome_fasta, contigs_sets_fasta{ic}, contig_set_names{ic}, quality, clipping_thrs, z_weights, distance,max_chromosome_dist);
            
            % Add overlaps, contigs and names to the lists
            [contig_all, overlap_all, name_hash_all, deltafiles_all] = update_values(contig_all, overlap_all, name_hash_all, deltafiles_all, ...
                                                                     contig_new, overlap_new, deltafile_new); 
                              
        end
 



    function [contig_all, overlap_all, name_hash_all, deltafiles_all] = ...
            update_values(contig_all, overlap_all, name_hash_all, deltafiles_all, ...
                          contig_new, overlap_new, deltafile_new)
    % Adds new contigs to contig_all and name_hash_all and all overlaps to overlap_all
                      
            % Loop through all contigs
            for i = 1:size(contig_new,1)
                % If a contig is new, add him to list.
                if ~isKey(name_hash_all, contig_new(i).name)
                    contig_all                        = [contig_all; contig_new(i)];
                    name_hash_all(contig_new(i).name) = size(contig_all,1);
                end
            end
            
            overlap_all    = [overlap_all overlap_new];
            deltafiles_all = [deltafiles_all deltafile_new];
    end


    function [adj_ref, names_ref, name_hash_ref, contig_ref, overlap_ref, deltafile] = align_contigs(chromosome_fasta, contig_set, set_name, quality, clipping_thrs, z_weights, distance,max_chromosome_dist)
    % Align an assembly against a chromosome?of the reference
        % INPUT
        % chromosome_fasta  Fasta file location of reference chomosome
        % contig_set        Mutiple fasta file with contigs
        % name              Contig_set_name
        % quality           Quality hash table
        % clipping_thrs     How much non-aligned overlap to allow (in number of nucleotides)
        % 
        % OUTPUT
        % adj_ref           Adjency matrix
        % names_ref         Names of contigs used in adj matrix
        % name_hash_ref     Name hash for for location of contig in struct
        % contig_ref        Contigs and their properties
        % overlap_ref       Overlaps of the contigs and their properties
        
            
%---------
        % Align the assembly against the chromosome
        [s,w] = unix(['nucmer -g 1000 ' chromosome_fasta ' ' contig_set]);
        if s ~= 0, w,error('Nucmer failed or is not findable by matlab'), end;
        % Filter 1-to-1 alignment allowing for rearrangements, minimum
        % identity is set to 50%, minimum alignment length to 200
        [s,w] = unix('delta-filter -i 96 -rq out.delta > out2.delta');
        if s ~= 0, w,error('delta-filter failed or is not findable by matlab'), end;
%---------       
        

%---------
        % Align the assembly against the chromosome
        %[s,w] = unix(['promer ' chromosome_fasta ' ' contig_set]);
        %if s ~= 0, w,error('Promer failed or is not findable by matlab'), end;
        % Filter 1-to-1 alignment allowing for rearrangements, minimum
        % identity is set to 50%, minimum alignment length to 200
        %[s,w] = unix('delta-filter -rqo 0 out.delta > out2.delta');
        %if s ~= 0, w,error('delta-filter failed or is not findable by matlab'), end;
%---------        
        
        
        % Make a human readable table
        %[s,w] = unix('show-coords -rcl out2.delta > out.coords');
        %if s ~= 0, w,error('show-coords failed or is not findable by matlab'), end;
        % Filter contigs that align with an end to the reference
        %[s,w] = unix(['/Volumes/linux-bastion/perl/filter_reference_alignments.pl out.coords ' num2str(clipping_thrs) ' > out.coords_filtered']);
        % Get filename
        %coords_file = [pwd '/out.coords'];
        % Read nucmer file
        %nuc = readnucmer({coords_file});

        % Read nucmer file
        [deltafile m] = readdelta([pwd '/out2.delta']);
        %nuc          = merge_alignment(nuc,'coords');
        deltafile     = merge_alignment(deltafile,'delta',m);
        
        
        % Transform coords table into the used datastructures
        
        
        if isequal(max_chromosome_dist, 'fuzzytiling')
            [adj_ref, names_ref, name_hash_ref, contig_ref, overlap_ref] = get_adj_matrix({deltafile}, {'reference',set_name}, quality, z_weights, 'clipping_thrs', inf, 'dove_tail', false,'edge_weight',distance);
        else
            [adj_ref, names_ref, name_hash_ref, contig_ref, overlap_ref] = get_adj_matrix({deltafile}, {'reference',set_name}, quality, z_weights, 'clipping_thrs', clipping_thrs, 'dove_tail', 'single','edge_weight',distance);
        end
        
    end




end
