function [am, As, pn] = assemble_path(assembly_path, contig, assemblies, deltafiles, deltafiles_ref, ref_chrom, overlap, varargin)
% Transform the assembly path in an alignment matrix
% 
% OUT
% am    Alignment matrix, one row per assembly
% As    Assembly names correspinding to the lines in am
% pn    Pseudo Nodes tracking: 
%       [Start_on_ref End_on_ref Start_in_alignMat End_in_alignMat]

    % Set defaults
    nrN = 50;
    
%    chromosome_size_estimate = 4000000;
    contigs_in_path          = contig(assembly_path); 
    chromosome_size_estimate = sum(arrayfun(@(x) x.size,contigs_in_path));

    % Initialize with the number of pseudonodes
    pn = zeros(sum(ismember({contig(assembly_path).assembly},'reference')),4);
    pni = 1;

    % Get VARARGIN user input
    if length(varargin) > 0
        while 1
            switch varargin{1}
                case 'chromosome_size_estimate'
                    chromosome_size_estimate = varargin{2};
                case 'clipping_thrs' 
                    clipping_thrs = varargin{2};
                case 'nrN'
                    nrN = varargin{2};
                case 'merge_overlapping'
                    merge_overlapping = varargin{2};
                otherwise
                    error(['Input ' varargin{2} ' is not known']);
            end
            if length(varargin) > 2
                varargin = varargin(3:end);
            else
                break;
            end
        end
    end

    % Read reference chromosome
    [ref_name, ref_seq] = fastaread(ref_chrom);

    % Relevant data for assembly
    rcontigs     = contig(assembly_path);
    rassemblies  = arrayfun(@(x) x.assembly, rcontigs, 'UniformOutput', false);
    rassembliesQ = arrayfun(@(x) x.assembly_quality, rcontigs, 'UniformOutput', false);
    [As index]   = unique(rassemblies);
    Asq          = rassembliesQ(index);
    
    % Overlap name array
    overlapR = arrayfun(@(x) x.R, overlap, 'UniformOutput', false)';
    overlapQ = arrayfun(@(x) x.Q, overlap, 'UniformOutput', false)';


    % Initialize alignment matrix
    am = zeros(length(As), chromosome_size_estimate,1);
    am = char(am);

    % Get deltafile names of R and Q contigs
    % size(deltafiles,2)
    Q  = arrayfun(@(x) x.Q, deltafiles, 'UniformOutput',false)';
    R  = arrayfun(@(x) x.R, deltafiles, 'UniformOutput',false)';
    Qr = arrayfun(@(x) x.Q, deltafiles_ref, 'UniformOutput',false)';
    Rr = arrayfun(@(x) x.R, deltafiles_ref, 'UniformOutput',false)';    
    


    % Load all contig sequences from disk
    assembly_sequences = load_assemblies(assemblies,ref_chrom);
    

    % Convert letters to numbers (A=1, C=2, T=3, G=4)


    % Check orientation fisrt contig
%     if contig(assembly_path(1)).orient ~= 53
%         error('First contig shoulod have 5'' - 3'' orientation');
%     end

    
    indels_previous = [];
    % Fill alignment matrix with appropiate contigs
    c = 1;
    while c <= length(assembly_path)
        
        fprintf('Adding contig %d to alignment matrix\n',c);
        
        assemname = contig(assembly_path(c)).assembly;
        
        if isequal(assemname,'reference')

            assemnr = find(ismember(As,assemname));
            
            assemname_next     = contig(assembly_path(c+1)).assembly;
            assemnr_next       = find(ismember(As,assemname_next)); 
            %contig_loc_nr_next = find(ismember(assemblies(:,1),assemname_next));
            %next_node_seq      = load_contig(contig(assembly_path(c+1)).name, contig_locs{contig_loc_nr_next});
            contig_set         = assembly_sequences(assemname_next);
            next_node_seq      = contig_set(contig(assembly_path(c+1)).name);
            
            
        else
            % Get a normal contig
            contig_loc_nr   = find(ismember(assemblies(:,1),assemname));
            %seq             = load_contig(contig(assembly_path(c)).name, contig_locs{contig_loc_nr});
            contig_set      = assembly_sequences(contig(assembly_path(c)).assembly);
            seq             = contig_set(contig(assembly_path(c)).name);
            assemnr         = find(ismember(As,assemname));
        end
        
        % Place first contig
        if c == 1
            
            % Add first contig to the alignment matrix
            if contig(assembly_path(c)).orient == 35
                am(assemnr,1:length(seq)) = seqrcomplement(seq);
            elseif contig(assembly_path(c)).orient == 53
                am(assemnr,1:length(seq)) = seq;
            end
                
            
            offset = 0;
            
            assemnr_previous   = assemnr;
            seq_previous       = seq;
            
            c = c + 1;
            
        else
            
            if isequal(assemname,'reference')
                
                % If a an alignment with the reference chromosome is hit
                % both the contig preceding the reference node
                % (predecessor) and the contig successing the reference
                % node (successor) have to be considered. 
                
                
                % Find the relevant alignments of the two contigs to the
                % reference chromosome (deltafiles)
                delta_pre = get_deltafile(deltafiles_ref,Rr,Qr,ref_name, contig(assembly_path(c-1)).name);
                delta_suc = get_deltafile(deltafiles_ref,Rr,Qr,contig(assembly_path(c+1)).name, ref_name);
                
                % Get the piece of reference chromosome aligned from
                % predecessor.S1 to successor.E1.
                ref_node_seq  = ref_seq(delta_pre.S1:delta_suc.E1);
                
                % Save refnode stats before normalizing deltafiles
                start_in_ref = delta_pre.E1 + 1;
                end_in_ref   = delta_suc.S1 - 1;   

                
                % adapt the deltafile
                % - Adjust LENR to be length(seq)
                % - Adjust alignment proporties to new size and start position of contig:
                %    predecessor.S1 = predecessor.S1 - predecessor.S1 + 1
                %    predecessor.E1 = predecessor.E1 - predecessor.S1 + 1
                %    successor.S1   = successor.S1 - predecessor.S1 + 1
                %    successor.E1   = successor.E1 - predecessor.S1 + 1 
                delta_pre.R    = contig(assembly_path(c)).name;
                delta_suc.R    = contig(assembly_path(c)).name;
                delta_pre.LENR = length(ref_node_seq);
                delta_suc.LENR = length(ref_node_seq);
                delta_pre.E1   = delta_pre.E1 - delta_pre.S1 + 1;   % Others are corrected 
                delta_suc.S1   = delta_suc.S1 - delta_pre.S1 + 1;   % for new startpoint
                delta_suc.E1   = delta_suc.E1 - delta_pre.S1 + 1;
                delta_pre.S1   = 1;                                 % new startpoint contig.
                
                
                % Split overlapping contigs and contigs with distance 0
                if ~merge_overlapping && (delta_suc.S1 - delta_pre.E1 <= 1)
                    
                    % Insertion sequence, 78 = N
                    insert = char(ones(1,nrN).*78);

                    % Create new reference contig, splitting overlappig part
                    ref_node_seq = [ref_node_seq(1:delta_pre.E1) insert ref_node_seq(delta_suc.S1:delta_suc.E1)];
                    
                    % Update alignment values
                    delta_pre.LENR = length(ref_node_seq);
                    delta_suc.LENR = length(ref_node_seq);
                    delta_suc.S1   = delta_suc.S1 + (delta_pre.E1 - delta_suc.S1) + nrN + 1;   % shift startpoint
                    delta_suc.E1   = delta_suc.E1 + (delta_pre.E1 - delta_suc.S1) + nrN + 1;   % and endpoint
                    
                end
                    
                
                % From here on treat them as normal contig alignments
                
                % Normalize deltafiles
                delta_pre = normalize_delta(delta_pre,contig(assembly_path(c-1)).name,seq_previous,...
                                            contig(assembly_path(c-1)).orient,...
                                            contig(assembly_path(c)).name,ref_node_seq, ...
                                            contig(assembly_path(c)).orient);
                                        
                [delta_suc,~,next_node_seq] = normalize_delta(delta_suc,contig(assembly_path(c)).name,...
                                                              ref_node_seq,contig(assembly_path(c)).orient,...
                                                              contig(assembly_path(c+1)).name,next_node_seq, ...
                                                              contig(assembly_path(c+1)).orient);
                
                                                          
                % add reference contig to alignment matrix
                [am new_offset]      = add_contig_to_matrix(am,delta_pre,offset,...
                                                            ref_node_seq,assemnr,...
                                                            assemnr_previous,indels_previous);
                % Process indels
                [am indels_previous] = process_indels(am, assemnr,...
                                                      assemnr_previous, delta_pre.indels,...
                                                      delta_pre.S1, indels_previous, offset);
                
                
                % fprintf('Adding contig %d to alignment matrix\n',c+1);
                
                % add contig successing reference contig to alignment matrix
                [am new_offset]  = add_contig_to_matrix(am,delta_suc,new_offset,...
                                                        next_node_seq,assemnr_next,...
                                                        assemnr,indels_previous);
                % Process indels
                [am indels_previous] = process_indels(am, assemnr_next, assemnr, ...
                                                      delta_suc.indels, delta_suc.S1, ...
                                                      indels_previous, new_offset);
                
                
                offset = new_offset;
                c      = c + 2; 
                
                assemnr_previous   = assemnr_next;
                seq_previous       = next_node_seq;
                deltafile_previous = delta_suc;
                
                
                % Save pseudonode stats; start and end position on reference  
                end_in_am    = new_offset+delta_suc.S2-1;
                start_in_am  = end_in_am - (end_in_ref - start_in_ref);
                pn(pni,:)    = [start_in_ref end_in_ref start_in_am end_in_am];
                pni          = pni + 1;
                
                
                % Process indels
                
                % Determine new offset based on successor contig
                
                
                % Get deltafile entry
                %deltafile      = get_deltafile(deltafiles_ref,Rr,Qr,ref_name, contig(assembly_path(c-1)).name);
                %deltafile_next = get_deltafile(deltafiles_ref,Rr,Qr,ref_name, contig(assembly_path(c+1)).name);
                %[S_current E_current S_previous E_previous indels] = get_startpos(deltafile, ref_name);
                %[S_next E_next S_current_next E_current_next indels] = get_startpos(deltafile_next, contig(assembly_path(c+1)).name);
                %seq = get_ref_seq(ref_chrom, S_current, E_current_next);
                %orient_ref = 53;
                %[S_current E_current S_previous E_previous indels seq] = normalize_delta(S_current, E_current, S_previous, E_previous, indels, orient_ref, contig(assembly_path(c-1)).orient, seq, seq_previous);
                
            else
            
                % Get deltafile entry
                deltafile = get_deltafile(deltafiles,R,Q,contig(assembly_path(c)).name, contig(assembly_path(c-1)).name);
                [deltafile, ~, seq] = normalize_delta(deltafile,contig(assembly_path(c-1)).name,seq_previous,contig(assembly_path(c-1)).orient,contig(assembly_path(c)).name,seq, contig(assembly_path(c)).orient);
                
                % Determine the position the next contig has to start relative
                % to the previous contig
                %[S_current E_current S_previous E_previous indels] = get_startpos(deltafile, contig(assembly_path(c)).name);

                % Add current contig to matrix
                [am new_offset] = add_contig_to_matrix(am,deltafile,offset,seq,assemnr,assemnr_previous,indels_previous);
                %am = add_contig_to_matrix(am,S_previous, E_previous, S_current, E_current, offset, seq, assemnr, assemnr_previous, indels_previous);


                % Introduce insertions in the two sequences
                % Get 'indel_pos_in_last' for processing of next alignment
                [am indels_previous] = process_indels(am, assemnr, assemnr_previous, deltafile.indels, deltafile.S1, indels_previous, offset);
                
                
                %offset = offset+deltafile.S1-deltafile.S2;
                offset = new_offset;
                c      = c + 1;
                
                assemnr_previous   = assemnr;
                seq_previous       = seq;
                deltafile_previous = deltafile;
            end
            
        
        end
        
        %if c > 1
        %    deltafile_previous = deltafile;
        %end
        
        
    end

    function seq = load_contig(contig_name, foldername)
        % Load contig from disk
        [~, seq] = fastaread([foldername contig_name]);
    end

    function assembly_sequences = load_assemblies(assemblies,ref_chrom)
        % Create container with all assemblies
        % key: assembly name
        % value: container with assemblies
        %       key: contig name
        %       value: sequence
        
        assembly_sequences = containers.Map;
        
        % Load assemblies
        for i = 1:size(assemblies,1)
            assembly_sequences(assemblies{i,1}) = load_sequences(assemblies{i,2});
        end
        
        % Load reference
        assembly_sequences('reference') = load_sequences(ref_chrom);
        
    end

    function seqs = load_sequences(assembly)
        % Load assembly contigs from disk and create hash table
        seqs = containers.Map;
        g = fastaread(assembly);
        for i = 1:length(g)
            header = regexp(g(i).Header,'\s','split');
            seqs(header{1}) = g(i).Sequence;
        end
    end

    function overlap = get_overlap(overlap, ovR, ovQ, name1, name2)
        % Get an entry from overlap from the overlap array for two given
        % names 'name1' and 'name2'
        
        loc = intersect(find(ismember(ovQ,name1)),find(ismember(ovR,name2)));
        if isempty(loc)
            loc = intersect(find(ismember(ovR,name1)),find(ismember(ovQ,name2)));
        end
        
        overlap = overlap(loc);
    end

    function deltafile = get_deltafile(deltafiles,R,Q,name1, name2)
        % Get an alignment delta for two contigs
        
        loc = intersect(find(ismember(Q,name1)),find(ismember(R,name2)));
        if isempty(loc)
            loc = intersect(find(ismember(R,name1)),find(ismember(Q,name2)));
        end
        
        deltafile = deltafiles(loc);
        
    end

    function [seq S_previous E_previous] = get_reference(overlap_entry, name_refnode)
        
        spl      = regexp(name_refnode,'_',split);
        startpos = split(2);
        endpos   = split(3);
        
        if isequal(overlap_entry.R, name_current)
            
            % Piece of the chromosome to put in between
            seq = get_ref_seq(chromosome, startpos, endpos);
            
            S_previous = overlap_entry.S2;
            E_previous = overlap_entry.E2;
            
        else
            error('Rederence node is always .R in overlap...');
        end
        

        
    end

    function [S_current E_current S_previous E_previous indels] = get_startpos(deltafile, name_current)
        % Get information from deltafile, taking into account waht is Q,
        % what is R and the orientation of the contig
        %
        %
        if isequal(deltafile.R, name_current)
            S_current  = deltafile.S1;
            E_current  = deltafile.E1;
            S_previous = deltafile.S2;
            E_previous = deltafile.E2;
            indels     = -deltafile.indels;
            
        elseif isequal(deltafile.Q, name_current)
            S_current  = deltafile.S2;
            E_current  = deltafile.E2;
            S_previous = deltafile.S1;
            E_previous = deltafile.E1;
            indels     = deltafile.indels; % Switch sign if previous contig is Query
        else
            error('contig not found in deltafile');
        end
        
    end

   

    function [Sr Er seqr] = get_reverse_complement(S, E, seq)
        % Get the start ans stop position in reverse complement format

        %       S           E
        % 3' -------------------- 5'
        
        %           ||       
        %           \/
        %       Er          Sr
        % 5' -------------------- 3'
        
        seqr = seqrcomplement(seq);
        Sr   = length(seq) - S + 1;
        Er   = length(seq) - E + 1;
        
    end

    function [S, E] = reverse_alignment(S,E)
        % Switch the coordinates of an alignment
                temp = S;
                S    = E;
                E    = temp;
    end

    function indels = reverse_indels(indels, A_length)
        % If the alignment is reversed the indels has to be reversed as
        % well
        
                % First indel offset is Alignment length - som(all offsets) + 1 
                indel_offset = A_length - sum(abs(indels)) + 1;
                % Set new offset on last position
                indels(end) = indel_offset;
                % Inverse indel list
                indels = indels(end:-1:1);                
    end



    function [am new_offset] = add_contig_to_matrix(am,d,offset,seq,assemnr,assemnr_R, indels_previous)
        % Add a new contig to the alignment matrix
        
        % Add sequence in the row of assemnr
        % Add the nucleotide from the alignment to S_previous plus the
        % offset in the matrix so far, 
        % The clipped off sequences are not included so the new sequence is
        % added from S_current to the end of the sequence. Next round could
        % clip off something from the end as well.
        
        inserts_in_R_below_start = sum(indels_previous < (d.S1 + offset) );
        startpoint = d.S1 + offset + inserts_in_R_below_start;
        
        % If a third or later contig overlaps the first position of the
        % alignment matrix, this has to be taken into account
        if startpoint <= 0
            
            BelowZeroCorrection = 0 - startpoint + 1;
            am(assemnr,startpoint+BelowZeroCorrection:length(seq)-d.S2+startpoint) = seq(d.S2+BelowZeroCorrection:end);

        else    
            am(assemnr,startpoint:length(seq)-d.S2+startpoint) = seq(d.S2:end);
        end
        %am(assemnr,S_previous+offset:length(seq)-S_current+S_previous+offset) = seq(S_current:end);
    
        % Clip the end of the previous contig
        new_E1 = d.E1 + length(indels_previous);
        am(assemnr_R,new_E1 + offset + 1:end) = ' ';
    
        % Process indels from previous alignment if they overlap with the
        % currently processed alignment
        for ins = 1:length(indels_previous)
            indel_pos                    = indels_previous(ins);
            if indel_pos > startpoint
                am(assemnr, 1+indel_pos:end) = am(assemnr, indel_pos:end-1);
                am(assemnr,   indel_pos)     = '.';
            end
        end
    
        new_offset = startpoint-d.S2;
    
    end

    function [am indel_locs] = process_indels(am, assemnr, assemnr_previous, indels,startpos_alignment, indels_previous, offset)
        % Put insertions into the alignment matrix
        % Remember what were the positions of insertions of the last contig
        % to use it for the next alignment
        
        %TODO rekenign houden met indels van previous alignment ( wanneer meerdere alignments overlappen ) 

        indel_locs   = [];
        indel_offset = 0;
        
        for inr = 1:length(indels)
            indel = indels(inr);
            
            if indel > 0
                % Insert is in current contig
                indel_pos                    = indel_offset+indel+offset+startpos_alignment;
                
                % Correct for indels from previous alignment
                indel_pos                    = indel_pos + sum(indels_previous <= indel_pos);
                
                % Indels from previous alignment that are further in the
                % sequence then this indels shift one position
                indels_previous(indels_previous > indel_pos) = indels_previous(indels_previous > indel_pos) + 1;
                
                am(assemnr, 1+indel_pos:end) = am(assemnr, indel_pos:end-1);
                am(assemnr,   indel_pos)     = '.';
                indel_locs                   = [indel_locs indel_pos];
                indel_offset                 = indel_offset + indel;
                
            elseif indel < 0
                % TODO moet deze indel niet ingevoerd worden in alle behalve 'assemnr'
                % insert is in previous contig
                indel                                 = -indel;
                indel_pos                             = indel_offset+indel+offset+startpos_alignment;
                indel_pos                             = indel_pos + sum(indels_previous < indel_pos);
                assemnr_ins                           = setdiff(1:size(am,1),assemnr);
                am(assemnr_ins, 1+indel_pos:end)      = am(assemnr_ins, indel_pos:end-1);
                am(assemnr_ins, indel_pos)            = '.';                
                
                % TODO Print the processing of indels to the screen
                %am(:,indel_pos-30:indel_pos+30)
                
                
                %am(assemnr_previous, 1+indel_pos:end) = am(assemnr_previous, indel_pos:end-1);
                %am(assemnr_previous, indel_pos)       = '.';                
                indel_offset                          = indel_offset + indel;
                
            else
                warning('number 0 found in indel sequence');
                return;
            end
        end
        
        % Add indel positions from previous alignment that are beyond the
        % startpoint of the alignment to the list of indels in this
        % sequence
        indel_locs = [indel_locs indels_previous(indels_previous > (startpos_alignment+offset) )];
        indel_locs = sort(indel_locs);
        
    end

    function seq = get_ref_seq(chromosome, startpos, endpos)
        % Get a piece of the reference chromosome
        [~, seq] = fastaread(chromosome);
        seq = seq(startpos:endpos);
    end

end
