function [adj, names_filtered, name_hash_filtered, contig_filtered, overlap_filtered] = get_adj_matrix(a, assembly, assembly_quality, z_weights, varargin)
% GET_ADJ_MATRIX Loads nucmer output, filtered by ~/perl/filter_nucmer.pl
%
% INPUT
% A                 DELTAFILE ARRAY
% ASSEMBLY:         n x 2 Cell array with assembly names, e.g. {'velvet' 's288c';'velvet' 'YJM789'}
% ASSEMBLY_QUALITY:  Hash table with assembly quality: key=assembly_name ;value=quality
%
% Jurgen Nijkamp
% 10-11-2009

% Some steps before you can use this script
    % Align met nucmer: nucmer --prefix=s288_chr11-velvetq21x2 /data2/jurgen/s288c/cns/s288-maq-chr11.fasta /data2/jurgen/CENPK/velvet/velvet27-cv4-filter-q21-x2-no-scaffold/contigs.fa
    % Filter met delta-filter: delta-filter -m s288_chr11-velvetq21x2.delta > delta-filter-m.coords
    % Filter met filter_nucmer.pl: ~/perl/filter_nucmer.pl s288c-velet.coords > s288c-velet-dove-tails.coords

    

    
    % Do a few checks
    if ~iscell(a)
        error('first input has to be of type cell');
    elseif ~iscell(assembly) || size(assembly,2) ~= 2
        error('Input value ASSEMBLY not correct');
    elseif ~strcmp(class(assembly_quality),'containers.Map')
        error('Input value ASSEMBLY_QUALITY has to be of type hash');
    end
    
    
    % Set defaults
    dove_tail = false;
    clipping_thrs = 10;
    containment_edge = false;
    
    
    % Get variable input
    for vi = 1:2:length(varargin)-1
        switch varargin{vi}
            case 'dove_tail'
                dove_tail = varargin{vi+1};
            case 'clipping_thrs'
                clipping_thrs = varargin{vi+1};
            case 'edge_weight'
                edge_weight = varargin{vi+1};
            case 'containment_edge'
                containment_edge = varargin{vi+1};
        end
    end
    
    % Give standard value to edge weight
    if ~exist('edge_weight','var')
        edge_weight = eps;
    end

    nr_of_nucmer_files = size(a,1);
    
    % Check input
    if size(assembly,1) ~= nr_of_nucmer_files || size(assembly,2) ~= 2
        error('assembly names do not correspond to file input');
    end
    
     
    % Get names of contigs and hash table
%    [names name_hash] = get_contig_names(a);
    
    
    % Get contig and overlap properties
    fprintf('Generating datastructures for overlaps and contigs\n');
    [contig overlap, names, name_hash] = generate_propertie_matrices(a, assembly, assembly_quality);
    %[contig, overlap] = get_props(a, contig, overlap, name_hash, assembly, assembly_quality);
    
    
    if ischar(dove_tail)
        %[overlap, contig, names, name_hash, weighted_z] = filter_dove_tails(overlap, contig, names, name_hash, clipping_thrs, weighted_z, dove_tail);
        [overlap_filtered, contig_filtered, names_filtered, name_hash_filtered] = filter_dove_tails(overlap, contig, names, clipping_thrs, dove_tail);

    else
        overlap_filtered    = overlap;
        contig_filtered     = contig;
        names_filtered      = names;
        name_hash_filtered  = name_hash;
    end
    
    % Filter contained edges
    if containment_edge
        [overlap_filtered, contig_filtered, names_filtered, name_hash_filtered] = filter_containment_edges(overlap_filtered, contig_filtered, names_filtered, clipping_thrs);
    end

    % Calculate edge-weights
    if isequal(edge_weight,'z-scores')
        % Calculate z-scores for edge weights
        weighted_z = weighted_z_scores(overlap_filtered, contig_filtered, name_hash_filtered, z_weights, overlap, contig);
        weighted_z(isnan(weighted_z)) = -8;
    else 
        % Add a constant value to the edges
        weighted_z = ones(size(overlap,2),2) .* edge_weight;
    end
    
    
    

    % Build adjency matrix 
    adj = build_adjency(overlap_filtered, names_filtered, name_hash_filtered, weighted_z);
    

    
%%%%%%%%%%% ********    SUBFUNCTIONS    ******* %%%%%%%%%%%
    
    function [m] = load_nucmer_filtered(filename)
        % load_nucmer_filtered Loads filtered nucmer files (see filter_nucmer.pl)
    
        % Load nucmer matrix filtered with filter_nucmer.pl ( ';' delimited)
        fid = fopen(filename);
        % Read with chr_nr and contig_nr determined by filter_nucmer.pl
        m = textscan(fid,'%d%d%d%d%d%d%n%d%d%n%n%s%s%d%d%d%d','Delimiter',';');
        fclose(fid);
    end
    
    function [contig, overlap] = get_props(b, contig, overlap, name_hash, assembly, assembly_quality)
        % get_props Get alignment properties from nucmer matrix
        % input: 13-column cell
        
        nr_overlaps = sum(cellfun(@(x) size(x,2), b));
        fprintf('%d overlaps to calculate\n',nr_overlaps);
        
        fprintf('Printing one dot per 10,000 overlaps\n');
        fprintf('Initialize datastructures: [');
        
        counter = 0;
        pos     = 1;
        NotSeenYet = ones(size(name_hash,1),1);
        
        for i = 1:size(b,1)   % Loop through all input files
            %bi = b(i,:);
            bi = b{i};
            assemblyi = assembly(i,:);
            
            %for j = 1:size(bi{1},1)
            for j = 1:size(bi,2)
                
                counter = counter + 1;
                
                if mod(counter,100) == 0
                    toc
                    fprintf('.');
                    tic
                end
                
                [contig, NotSeenYet] = get_contig_props(bi,contig,name_hash,j,assemblyi,assembly_quality, NotSeenYet);
                overlap = get_overlap_props(bi,overlap,j,pos);
                pos = pos+1;
            end
        end
    end

    function overlap = get_overlap_props(d,overlap,j,pos)
        
        % j   : Position in deltafile
        % pos : Position in overlap struct
        
        % Get overlap properties
        %pos = size(overlap,2) + 1;
        
        
        overlap(pos).R = d(j).R;
        overlap(pos).Q = d(j).Q;
        overlap(pos).S1 = d(j).S1;
        overlap(pos).E1 = d(j).E1;
        overlap(pos).S2 = d(j).S2;
        overlap(pos).E2 = d(j).E2;
        overlap(pos).LEN1 = abs(overlap(pos).E1 - overlap(pos).S1) + 1;
        overlap(pos).LEN2 = abs(overlap(pos).E2 - overlap(pos).S2) + 1;
        overlap(pos).IDY = (overlap(pos).LEN1 - d(j).error)/overlap(pos).LEN1;
        overlap(pos).COVR = (1-(d(j).LENR-overlap(pos).LEN1)/d(j).LENR)*100;
        overlap(pos).COVQ = (1-(d(j).LENQ-overlap(pos).LEN2)/d(j).LENQ)*100;
        
        
%         % Get overlap properties
%         pos = size(overlap,2) + 1;
%         overlap(pos).R = bi{12}{j};
%         overlap(pos).Q = bi{13}{j};
%         overlap(pos).S1 = bi{1}(j);
%         overlap(pos).E1 = bi{2}(j);
%         overlap(pos).S2 = bi{3}(j);
%         overlap(pos).E2 = bi{4}(j);
%         overlap(pos).LEN1 = bi{5}(j);
%         overlap(pos).LEN2 = bi{6}(j);
%         overlap(pos).IDY = bi{7}(j);
%         overlap(pos).COVR = bi{10}(j);
%         overlap(pos).COVQ = bi{11}(j);
    end


    function [contig, NotSeenYet] = get_contig_props(d,contig,name_hash,j,assemblyi,assembly_quality, NotSeenYet)
        % Get contig properties
        
        pos = name_hash(d(j).R);
        if NotSeenYet(pos) == 1
            % Reference
            pos = name_hash(d(j).R);
            contig(pos).size = d(j).LENR;
            %if size(bi,2) > 13
            %    contig(pos).chromosome = bi{14}(j);
            %    contig(pos).number = bi{16}(j);
            %else
                contig(pos).chromosome = 0;
                contig(pos).number = 0;
            %end              
            contig(pos).assembly = assemblyi{1};
            contig(pos).assembly_quality = assembly_quality(assemblyi{1});
            NotSeenYet(pos) = 0;
        end
        
        
        pos = name_hash(d(j).Q);
        if NotSeenYet(pos) == 1        
            % Query
            pos = name_hash(d(j).Q);
            contig(pos).size = d(j).LENQ;
            %if size(bi,2) > 13
            %    contig(pos).chromosome = bi{15}(j);
            %    contig(pos).number = bi{17}(j);
            %else
                contig(pos).chromosome = 0;
                contig(pos).number = 0;
            %end 
            contig(pos).assembly = assemblyi{2};
            contig(pos).assembly_quality = assembly_quality(assemblyi{2});  
            NotSeenYet(pos) = 0;
        end
        
        

%         % Reference
%         pos = name_hash(bi{12}{j});
%         contig(pos).size = bi{8}(j);
%         if size(bi,2) > 13
%             contig(pos).chromosome = bi{14}(j);
%             contig(pos).number = bi{16}(j);
%         else
%             contig(pos).chromosome = 0;
%             contig(pos).number = 0;
%         end              
%         contig(pos).assembly = assemblyi{1};
%         contig(pos).assembly_quality = assembly_quality(assemblyi{1});
%         
%         % Query
%         pos = name_hash(bi{13}{j});
%         contig(pos).size = bi{9}(j);    
%         if size(bi,2) > 13
%             contig(pos).chromosome = bi{15}(j);
%             contig(pos).number = bi{17}(j);
%         else
%             contig(pos).chromosome = 0;
%             contig(pos).number = 0;
%         end 
%         contig(pos).assembly = assemblyi{2};
%         contig(pos).assembly_quality = assembly_quality(assemblyi{2});
    end
        
    function [names name_hash] = get_contig_names(a)
        
        Rs = [];
        Qs = [];
        
        for i = 1:size(a,1)
            fprintf('Getting names for alignment file %d / %d\n', i, size(a,1));
            ar = rmfield(a{i},(setdiff(fieldnames(a{i}),'R')));
            Rs = [Rs; squeeze(struct2cell(ar))];
            aq = rmfield(a{i},(setdiff(fieldnames(a{i}),'Q')));
            Qs = [Qs; squeeze(struct2cell(aq))];
        end
        
        fprintf('Getting unique contig names\n', i, size(a,1));
        names = vertcat(Rs,Qs);
        names = unique(names);
        fprintf('Make hash table for contig names\n', i, size(a,1));
        name_hash = make_hash(names);
    end

          
    function adjmatrix = build_adjency(overlap, names, name_hash, weighted_z)
        %Build main adjency matrix
        adjmatrix = sparse(length(names),length(names));
        
        overlap_cell = struct2cell(overlap);
        R = squeeze(overlap_cell(2,:,:));
        Q = squeeze(overlap_cell(1,:,:));
        fprintf('Adding edges to adjency\n');  
        fprintf('1000 edges of %d processed per dot [',size(overlap,2));  
        
        for j = 1:size(R,1)
            if mod(j,1000) == 0
                fprintf('.');
                if mod(j,50000) == 0
                    fprintf('\n');
                end
            end
            
            Rj = name_hash(R{j});   
            Qj = name_hash(Q{j});
            adjmatrix(Rj,Qj) = weighted_z(j,2);
            adjmatrix(Qj,Rj) = weighted_z(j,1);
        end
        fprintf('] done\n');
    end

    function [contig, overlap, names, name_hash, LENR, LENQ] = generate_propertie_matrices(b, assembly, assembly_quality)
        
        nr_overlaps = sum(cellfun(@(x) size(x,2), b));
        
        nr_overlaps = sum(cellfun(@(x) size(x,2), b));
        R = cell(1,nr_overlaps);
        Q=R; S1=R; E1=R; S2=R; E2=R; LEN1=R; LEN2=R; IDY = R; COVR=R; COVQ=R; ERROR=R; LENR=R; LENQ=R;Rassembly=R;Qassembly=R;Rqual=R;Qqual=R;
        
               
        i1 = 1;
        if nr_overlaps ~= 0
			for ovi = 1:size(b,1)
				
				if isempty(b{ovi})
					continue;
				end
				
				i2 = size(b{ovi},2) + i1 - 1;
				R(i1:i2)        = {b{ovi}.R};
				Q(i1:i2)        = {b{ovi}.Q};
				S1(i1:i2)       = {b{ovi}.S1};
				E1(i1:i2)       = {b{ovi}.E1};
				S2(i1:i2)       = {b{ovi}.S2};
				E2(i1:i2)       = {b{ovi}.E2};
				LENR(i1:i2)     = {b{ovi}.LENR};    % Contig lengths
				LENQ(i1:i2)     = {b{ovi}.LENQ};
				ERROR(i1:i2)    = {b{ovi}.error};
				Rassembly(i1:i2)= assembly(ovi,1);
				Qassembly(i1:i2)= assembly(ovi,2);
				Rqual(i1:i2)    = {assembly_quality(assembly{ovi,1})};
				Qqual(i1:i2)    = {assembly_quality(assembly{ovi,2})};
				i1 = i2 + 1;
			end
			
			LEN1 = abs(cell2mat(E1) - cell2mat(S1)) + 1; % Alignment lengths
			LEN2 = abs(cell2mat(E2) - cell2mat(S2)) + 1;
			IDY  = (double(LEN1) - double(cell2mat(ERROR)))./double(LEN1) * 100;
			COVR = (1-(cell2mat(LENR)-LEN1)./cell2mat(LENR)).*100;
			COVQ = (1-(cell2mat(LENQ)-LEN2)./cell2mat(LENQ)).*100;
			
			overlap = struct('Q',Q,'R',R,'S1',S1,'E1', E1, 'S2', S2, 'E2', E2, 'LEN1', num2cell(LEN1), 'LEN2', num2cell(LEN2), 'IDY', num2cell(IDY), 'COVR', num2cell(COVR), 'COVQ', num2cell(COVQ),'LENR',LENR, 'LENQ',LENQ);
		else
			overlap = struct('Q',[],'R',[],'S1',[],'E1', [], 'S2', [], 'E2', [], 'LEN1', [], 'LEN2', [], 'IDY', [], 'COVR', [], 'COVQ', [],'LENR',[], 'LENQ',[]);
		end

        % Merge R and Q
        names       = [R Q];
        LEN         = [LENR LENQ];
        Assembly    = [Rassembly Qassembly]; 
        Quality     = [Rqual Qqual];
        
        % Get uniques
        [names ui]  = unique(names);
        LEN         = LEN(ui);
        Assembly    = Assembly(ui);
        Quality     = Quality(ui);
        po          = num2cell(repmat(0,1,length(ui)));
        contig      = struct('name',names,'size',LEN,'chromosome',[],'number',[], 'assembly', Assembly, 'assembly_quality', Quality,'preferred_orient',po);
        
        % Make hash table for positions in the graph
        names     = names';
        contig    = contig';
        name_hash = make_hash(names);
        
    end
end