function [longest_path weigth] = longest_path_tabu(G, start_node, end_node, quit_threshold, tabu_time, minimum_improvement)
%LONGEST_PATH_TABU Find the longest path between two nodes in a graph  
%   [longest_path weigth] = longest_path_tabu(G, start_node, end_node,
%                           quit_threshold, tabu_time, minimum_improvement)
%
%   This function tries to find the longest path using a two step algorithm:
%
%   1.  Find initial path with Dijkstra on inverted adjency matrix
%       The Dijkstra algorithm is known for finding the shortest path. Here
%       it is applied to find an initial 'longest path'. This is done by
%       applying Dijkstra on the inverted weights w of the adjency matrix G.
%       The weights are inverted by substrating them from the maximimum
%       value M in G. Since the maximum value will become zero using this 
%       transformation, which means 'no edge', a weight of 1 is added to 
%       all nodes. inv_weights = M - w + 1
%
%   2.  The path found by Dijkstra on the inverted weights may not by the
%       optimal longest path, but a local optimum instead. To get closer to 
%       the global optimum a TABU optimization is applied to the path. TABU
%       starts with systematically applying a change to the path that
%       yields the most improvement in path length. There are three types
%       of changes that can be applied:
%       1. A node can be removed by directly bypassing this node
%       2. An extra node can be added in between two nodes
%       3. A node can be replaced by another node connecting the same
%          neighbour nodes in the path
%       After the change has been applied, the inverse of the change is
%       stored in the TABU list to avoid entrapment in cycles of the same
%       change. After a certain number of cycles ('tabu_time')the change is
%       removed from the TABU list again and inversion of the change is
%       allowed again. The algorithm proceeds until for a certain number of
%       changes (quit_threshold) no improvement is seen compared to the 
%       best path found so far.
%
%       INPUT
%       G                   : Adjency matrix
%       start_node          : First node in the path
%       end_node            : Last node in the path
%       quit_threshold      : Max number of changes without improvement
%       tabu_time           : Number of rounds a move cannot be inverted
%       minimum_improvement : Minimum improvement a change should to the
%                             path should add to the weight (or distance)
%                             This can be a negative ot positive number
%
%       OUTPUT
%       longest_path        : Longest path found by TABU
%       weigth              : Weigth (or distance) of the path
%
%
%       AUTHORS             : Wynand Winterbach en Jurgen Nijkamp,
%                             Delft University of Technology, NL
%       DATE                : November 2009
%       
%       REFERENCE           : Glover F.," Future paths for Integer 
%                             Programming and Links to Artificial 
%                             Intelligence", Computers and Operations 
%                             Research, 5:533-549, 1986. 
%
%   See also dijkstra

    % CONSTANTS
    NO_CHANGE = -1;     % No change at all
    MOVE = 0;           % Interchange of two nodes
    REMOVE = 1;         % Removal of a node
    ADD = 2;            % Addition of a node
    
    % Set defaults, if not given in input
    if nargin < 6
        minimum_improvement = -inf;
        if nargin < 5
            tabu_time = 5;
            if nargin < 4
                quit_threshold = 50;
            end
        end
    end
    
    % Get initial local optimum using Dijkstra on inverted weight matrix
    path = get_initial_solution(G);
    
    % Optimize solution using TABU local search
    [longest_path weigth] = tabu(path, G, quit_threshold, tabu_time);
    
    function path = get_initial_solution(G)
        % We want to use Dijkstra to find the longest paths.
        % Since Dijkstra doesn't work with negative values,
        % we convert our weights to M - w + 1, where M is the
        % maximum weight in the graph, w is a arbitrary weight.
        % We add 1, since if w = M, we'll end up with a weight of
        % zero.
        inverted_G = invert_adjency_matrix(G);

        [D pred] = dijkstra(inverted_G, start_node);
        
        if isinf(D(end_node))
            error('No path from start to end node found');
        end
        
        path = build_path(pred, end_node);   % convert predecessor list to path 
    end

    function [P, Pall, Wall] = find_all_paths(G, begin, finish, max_depth, P, W, Pall, Wall, depth)
        % Find all paths from begin to finish recursively
        % before running this function, initialize global empty cell arrays called
        % Pall and Wall
        % 
        if ~exist('P', 'var'),
            P = [];
        end
        if ~exist('W', 'var'),
            W = [];
        end
        if ~exist('Pall', 'var'),
            Pall = {};
        end
        if ~exist('Wall', 'var'),
            Wall = {};
        end
        if ~exist('depth', 'var'),
            depth = 0;
        end

        if depth > max_depth
            return
        end

        %global Pall
        %global Wall

        P = [P begin];

        % fprintf('Node is: %d\n',begin)
        % pause



        if begin == finish        % Target node is found!
           if isempty(Pall)       % Pall saves all paths to target node
               Pall = {P};        % Add path to Pall 
               Wall = {W};        % Add weigths to Wall 
           else
               Pall = [Pall; P];  % Add path to Pall   
               Wall = [Wall; W];  % Add weigths to Wall 
           end
           return
        end

        nodes_connected = find(logical(G(begin,:)));  % Find the nodes connected to begin

        for i = 1:length(nodes_connected)          % Extend to all nodes connected to begin
            next_node = nodes_connected(i);        % Extend to node i
            if sum(next_node == P) == 0            % If the node i is in the path, the skip to prevent cycles
                [P2 Pall Wall] = find_all_paths(G, next_node, finish, max_depth, P, [W full(G(begin,next_node))], Pall, Wall, depth + 1);  % Find path from next_node
            end
        end
        
        return

    end
  
%TAKEN OUTSIDE OF TABU
 %   function inverted_G = invert_adjency_matrix(G)
 %       inverted_G = max(max(G)).*ones(size(G)) - G + 1;
 %       inverted_G(G==0) = 0;
 %   end
    
    function [paths, weights] = filter_path(paths, weights, unvalid_path, complete_path)  
        % Filters out the 'unvalid_path' from the paths list 'paths'
        % Also filters out the correspondgin weights.
        % INPUT
        % paths           : Modification paths
        % weigths         : Corresponding weights
        % unvalid_path    : The 2 or 3 nodes that are the part of the 
        %                   part of the path that is going to be changes
        % complete_path   : The complete path as it is now
        
        % Filter out the path that is no modification, but is the way it is now
        valid_paths_index = cellfun(@(x) ~isequal(x,unvalid_path), paths);
        paths = paths(valid_paths_index);
        weights = weights(valid_paths_index);   
        
        % Remove modification paths that introduce a node that is already
        % in the complete path
        valid_paths_index = logical(ones(length(paths),1));
        for i = 1:length(paths)
            p = paths{i};
            if length(p) == 3
                if ismember(p(2),complete_path)
                    valid_paths_index(i) = false;
                end
            end
        end
        paths = paths(valid_paths_index);
        weights = weights(valid_paths_index);   
    end

    function [best_path, best_weight_change, best_change_type] = get_best(paths, weight_changes, change_types)
        % INPUT
        % paths            : a list of paths
        % weights          : a list of weigth changes corresponding to the paths
        % change_types     : corresponding to the path
        % OUTPUT
        % best_path        : Best path based on weight
        % best_weight      : Weights corresponding to edges of best path
        % best_change_type : Change type corresponding to best path
        
        if size(paths,1) < 1
            best_path           = paths;
            best_weight_change  = weight_changes;
            best_change_type    = change_types;
        else        
            
            [best_weight_change, best_row_index] = max(weight_changes);
            best_path                            = paths{best_row_index};
            best_change_type                     = change_types(best_row_index);
        end
        
        
    end

    function [valid_paths, valid_weights, valid_change_types] = filter_tabu(tabu_list, tabu_keys, paths, weights, change_types)
        % Filter out paths in 'paths' (and thus also corresponding entries
        % in 'weights' and 'change_types') that have entries in the tabu
        % list 'tabu_list'. 'tabu_keys' contains the information in
        % 'paths' and 'change_types', but in a format that can be used to
        % query 'tabu_list' directly. From this query, we'll get a list of
        % index values of paths which are found in the tabu list and which
        % must therefore be filtered out.
   
        index_of_keys_in_tabu_list = ~(sum(ismember(tabu_keys, tabu_list),2) == 5);
        
        valid_paths        = paths(index_of_keys_in_tabu_list,:);
        valid_weights      = weights(index_of_keys_in_tabu_list,:);
        valid_change_types = change_types(index_of_keys_in_tabu_list,:);

    end

    function path = apply_move(path, move, change_type)
        % Chage path according to the given move
        % INPUT
        % path          : array with numbers from start to end node
        % move          : vertices to change
        % change        : change_type
        % change_vertex : The vertex that is going to be affected
        
        switch change_type 
            case ADD
                v1 = find(path == move(1)); % Index of v1
                v3 = find(path == move(3)); % index of v3
                path = [path(1:v1) move(2) path(v3:end)];
            case REMOVE
                v1 = find(path == move(1)); % Index of v1
                v2 = find(path == move(2)); % index of v3                
                path = [path(1:v1) path(v2:end)];
            case MOVE
                v1 = find(path == move(1)); % Index of v1
                v3 = find(path == move(3)); % index of v3
                path = [path(1:v1) move(2) path(v3:end)];
        end 
    end

    function weight = get_path_weight(path, G)
        % Calculate weight of a path by summing the edges
        
        weight = 0;
        
        for i = 1:length(path)-1
            weight = weight + G(path(i),path(i+1));
        end
    end

    function list = get_tabu_keys(paths, change_vertex,all_change_types)
        % Returns a list of keys in the format:
        % [ADD,    v1, v3, 0,  v4]
        % [REMOVE, v1, v3, v2, 0 ]
        % [MOVE,   v1, v3, v2, v4]
        %
        % Column 4 gives the node that is removed whilst column 5 gives the 
        % newly added node. For additions, column 4 is 0, since we're not 
        % removing anything. For removals, column 5 is 0, since we're not
        % adding anything. For interchanges, column 4 is the node to
        % removed and column 5 is the new node which takes the node in
        % column 4's place.
        
        % Initialize return list
        list = zeros(length(paths),5);
        
        % First column gets the change types (ADD, REMOVE or MOVE)
        list(:,1) = all_change_types;
        
        % Loop through all paths in 'paths' and fill the list with the
        % corresponding tabu keys
        for i = 1:length(paths)
            path = paths{i};
            % Column 2 gets the starting node (v1) for each change
            list(i,2) = path(1);
            % Column 3 gets the ending node (v3) for each change
            list(i,3) = path(length(path));
            % Column 4 gets the added node in case of ADD else 0
            if all_change_types(i) ~= ADD
                list(i, 4) = change_vertex;
            else
                list(i, 4) = 0;
            end
            % Column 5 gets the removed vertex in case of REMOVE else 0
            if all_change_types(i) ~= REMOVE
                list(i, 5) = path(2);
            else
                list(i, 5) = 0;
            end
        end
    end

    function [change_type] = get_change_types(paths)
        % Returns change type for Removals and Interchanges by looking at
        % the path lengths in 'paths'. Additions are already handled in
        % find_modification_paths.
        
        change_type = zeros(length(paths), 1);
        for i = 1:length(paths)
            if length(paths{i}) == 2
                change_type(i) = REMOVE;
            else
                change_type(i) = MOVE;
            end
        end
    end

    function [paths, weight_changes, change_type] = find_modification_paths(path, G, path_index)
        % Return a list of all paths starting at path(path_index) such that
        % these paths either extend, shorten, or reroute the path described
        % by 'path'. Extension allows the addition of a single node,
        % shortening allows the removal of a single node and rerouting
        % allows a single node to be changed (such that we still end up
        % with a valid path).
        %
        % path:       a list of vertices giving a valid path in the graph G
        % G:          a graph
        % path_index: the first vertex of every change in 'paths'
        %
        % returns
        % paths:      a list of paths which can be used to extend, shorten 
        %             or reroute the path in 'path'
        % weights:    a list of list of edge weights of the paths given in 
        %             'paths' - the list of edge weights at index i
        %             corresponds to the path at index i in 'paths'
        % change_type: a list of constants which specifies the type of
        %              changes represented by the paths in 'paths'. The
        %              constant at index i corresponds to the path at index
        %              i in 'paths'.
        
        
        % Node v1
        v                   = path(path_index);
        % Node v2
        next_v              = path(path_index + 1);
        
        % Find all paths from v1 to v2 with max length of 2 edges
        % This includes all possible additions
        [P, paths, weights] = find_all_paths(G, v, next_v, 2);
        
        % Filter out the path v1-v2, since we only want additions
        [paths, weights]    = filter_path(paths, weights, [v, next_v], path);
        
        % Get the change of weight as compared to the previous path
        path_weights   = cellfun(@(x) sum(x),weights);
        weight_changes = path_weights - G(v, next_v);
        
        % Create entries in the "change_type" list corresponding to the
        % type of the change (only addidions here)
        change_type         = ADD * ones(length(paths), 1);
       
        % If are at the second last node of 'path' then we can't do
        % anything else since we can only do additions in that case, but if
        % not then we can possibly do node removal or interchange
        weights = [];
        if path_index < length(path) - 1 
            
            % Node v3
            next_next_v = path(path_index + 2);
            
            % Find all paths between v1 and v3 of at most 2 edges
            [P, alternative_paths, alternative_weights] = find_all_paths(G, v, next_next_v, 2);
            
            % Fiter ou the path v1-v2-v3, because that is already part of
            % our current path
            [alternative_paths, alternative_weights] = filter_path(alternative_paths, alternative_weights, [v, next_v, next_next_v], path);        
            
            paths       = [paths; alternative_paths];
            weights     = [weights; alternative_weights];
            change_type = [change_type; get_change_types(alternative_paths)];
            path_weights   = cellfun(@(x) sum(x),weights);
            weight_changes = [weight_changes; path_weights - (G(v, next_v) + G(next_v, next_next_v))];
        end
    end

    function [move, move_weight_change, change_type, change_vertex] = next_single_move(tabu_list, path, G, path_index)
        % 
        % This is the vertex that undergoes change:
        %     move:   [v1, v2, v3] -> [v1, v4, v3]     change_vertex == v2
        %     remove: [v1, v2, v3] -> [v1, v3]         change_vertex == v2
        %     add:    [v1, v3]     -> [v1, v2, v3]     change_vertex == v3
        %     (we ignore this vertex)
        change_vertex = path(path_index + 1);
        
        % Find all possible changes that we can apply to the path
        % with path(path_index) as the first vertex
        [all_paths all_weight_changes, all_change_types] = find_modification_paths(path, G, path_index);
        
        % Generate tabu keys for all the possible modifications that have been 
        % found by find_modification
        tabu_keys = get_tabu_keys(all_paths, change_vertex,all_change_types); 
        
        % Check wether the keys are in the tabu list and hence not valid
        % ande filter out tabu modifications
        [valid_paths, valid_weights_changes, valid_change_types] = filter_tabu(tabu_list, tabu_keys, all_paths, all_weight_changes, all_change_types);
        
        % Get the modification that has the highest weight score
        [move, move_weight_change, change_type] = get_best(valid_paths, valid_weights_changes, valid_change_types);
    end
    
    function [move, move_weight_change, change_type, change_vertex] = next_move(tabu_list, path, G, minimum_improvement)
        % Enumerate all nodes in 'path' and for each node 'v' find the 
        % modification that maximizes the weight score, with 'v' as the
        % starting node of the modification. Out of all these modifications
        % select the modification that maximizes the weight score for
        % 'path'. 

        
        move                 = [];
        move_weight_change   = minimum_improvement; 
        change_type          = NO_CHANGE;

        
        for v = 1:length(path)-1,
            [single_move, single_move_weight_change, single_change, change_vertex] = next_single_move(tabu_list, path, G, v);
            
            % Sum cannot operate on a single cell array, so check
            if isempty(single_move_weight_change)
                continue
            end
                        
            if single_move_weight_change > move_weight_change
                move                 = single_move;
                move_weight_change   = single_move_weight_change;
                change_type          = single_change;
            end
        end
        
        if isempty(move)
            move = -1;
            change_vertex = -1;
            
        end
        
    end

    function tabu_list = update_tabu_list(tabu_list, move, change_type, change_vertex, tabu_time)
        % [ADD,    v1, v3, 0,  v4]
        % [REMOVE, v1, v3, v2, 0 ]
        % [MOVE,   v1, v3, v2, v4]
        
        % Update tabu_time
        if ~isempty(tabu_list)
            tabu_list(:,end) = tabu_list(:,end) - 1;
        end
            
        % Add new tabu keys
        switch change_type 
            case ADD
                tabu_list = [tabu_list; [REMOVE move(1) move(3) move(2) 0 tabu_time]];
            case REMOVE
                tabu_list = [tabu_list; [ADD move(1) move(2) 0 change_vertex tabu_time]];
            case MOVE
                tabu_list = [tabu_list; [MOVE move(1) move(3) move(2) change_vertex tabu_time]];
        end 
     
       %  Remove keys from list that have 'tabu_time' = zero
        tabu_list(tabu_list(:,end) == 0,:) = [];
        
        
    end

    function [best_path best_weight] = tabu(path, G, quit_threshold, tabu_time)
        % INPUT
        % path          : Path found by some greedy method
        % G             : Adjency matrix
        % quit_treshold : Maximum number of times a change can be applied
        %                 to the path without improving the best path found
        %                 so far.
        
        path_weight = get_path_weight(path, G);
        
        
        % The inverse of the changes applied to the path are stored in 
        % 'tabu_list' to prevent the algorithm from constantly applying the
        % same change.
        tabu_list = [];
        
        % A count of the number of modifications applied without
        % improvement of the the best path found so far
        number_of_moves_without_improvement = 0;
        
        % Best path found
        best_path = path;
        best_weight = path_weight;
        current_weight = path_weight;
        current_path = path;
        
              
        % Apply changes to the path until the path does not improve
        % anymore. No improvement means in this case is reached when the
        % number of changes applied to the path without improvement exceeds
        % the specified 'quit_treshold'
        while number_of_moves_without_improvement < quit_threshold,
            
            % Find the move that improves the graph the most, but does not
            % occur in 'tabu_list'
            [move, move_weight_change, change_type, change_vertex] = next_move(tabu_list, current_path, G, minimum_improvement);
             
            % If there are no moves possible, break
            if isequal(move,-1)
                break
            end
            
            
            % It can be that there is no change possible to the path (it is
            % a simple path or all possible changes are in 'tabu_list'
            if change_type == NO_CHANGE
                % fprintf('No change..\n');
                break
            end
            
            % Update the weight and path accoding to the move found by
            % 'next_move'
            current_weight = current_weight + move_weight_change;
            current_path = apply_move(current_path, move, change_type);
            tabu_list = update_tabu_list(tabu_list,move,change_type, change_vertex, tabu_time);
            
            %fprintf('number_of_moves_without_improvement %d\n',number_of_moves_without_improvement);
            %fprintf('current_weight %d\n',current_weight);
            %fprintf('move %d\n',move);
            
            
            %current_path
            
            
            % If after applying the move the total weight is higher then
            % the current optimum, then update the optimal path and weight
            if round(10000*current_weight) > round(10000*best_weight)
                best_weight = current_weight;
                best_path = current_path;
                number_of_moves_without_improvement = 0;
            else
                number_of_moves_without_improvement = number_of_moves_without_improvement + 1;
            end
                
        end
    end
end

