function maia(Assemblies_file,reference,varargin)
% MAIA Multiple Assembly IntegrAtor
%
% [] = maia( ASSEMBLIES, REFERENCEGENOME, VARARGIN )
%
% INPUT:
% ASSEMBLIES  Tab delimited file with on every row:
% <Name>    <Fasta File>    <Quality score>
% 
% REFERENCEGENOME (fasta)
% '/home/users/ref.fasta';
% 
% OPTIONAL PARAMETERS
% PARAMETER             | VALUE             | DEFAULT   | DESCRIPTION
% -------------------------------------------------------------------------
%                       |                   |           |
% MAIN OPTIONS          |                   |           |    
% 'alignlen'            | [0, inf]          | 75        | Minimum length of alignment between contigs (delta-filter)
% 'ident'               | [0, 100]          | 85        | Minimum identity of alignment (delta-filter) 
% 'mapfilter'           | {-r,-q,-1,-m,-g}  | -rq       | See delta-filter documentation
% 'clipping_thrs'       | [0, inf]          | 10        | Max non-aligned overhand
% 'ref_node_assemblies' | [1, #assemblies]  | 1         | Which assembly to use to ad pseudonodes 
% 'containment_edge'    | bool              | true      | Filter out containment edges
%                       |                   |           |
% POST PROCESSING       |                   |           |
% 'extend_ends'         | true or false     | true      | Greedy extend the ends of the obtained assembly path 
% 'max_ref_node_size'   | [-inf, inf]       | inf       | Maximimum size of a pseudo node 
% 'min_alen_extend'     | [10,inf]          | 400       | Minimum alignment length for the greedy extension 
% 'min_ident_extend'    | [0,100]           | 99        | Minimum identity for the greedy extension 
% 'ReplacePseudo'       | boolean           | true      | Replace bases coming from pseudonodes by 'N'
% 'max_backtrack_depth' | [1, inf]          | 1         | When pseudonode is removed, the algorithm tracks back this far
% 'merge_overlapping'   | true or false     | false     | Merge contigs that overlap on the reference
%
%
% 
% 
% 
% EXAMPLE
% 
% >> MyInputAssemblies = '/home/user/InputAssemblies.txt';
% >> MyRefGenome       = '/home/user/ref.fasta';
% 
% >> maia(MyInputAssemblies, MyRefGenome,'containment_edge',false, 'clipping_thrs', 200)
% 
% 
% RESULT:
% 1. maia_assembly.fa     Fasta file with integrated contigs
%
% 2. ref_contig#.xgmml    One Cytoscape graph per reference chromosome
% 
% 
% Author: Jurgen Nijkamp <J.F.Nijkamp@tudelft.nl>
% Date:   5 October 2010
%
%
% REFERENCE
% Jurgen Nijkamp, Wynand Winterbach, Marcel van den Broek, Jean-Marc Daran, Marcel Reinders, and Dick de Ridder 
% Integrating genome assemblies with MAIA
% Bioinformatics 2010 26: i433-i439.


% Read input assemblies
    Assemblies = loadassemblies(Assemblies_file);

    
% Load reference
    ref = fastaread(reference);

    
% 1. Set Defaults

    % Paramters for delta-filter filtering
    mapfilter           = '-rq';
    alignlen            = 75;
    ident               = 85;

	% Which assembly do you want to align to the reference to connect pseudonodes
	Assemblies_for_refnodes = [1];
    
    % Extend the ends of the contigs (before split and extend)
    endextend = true;

	% Define weights, which are used for z-score combination
	% The following weights have to be defined:
	% z_weights(1) = Contig length
	% z_weights(2) = Alignment length
	% z_weights(3) = Non aligned overlap
	% z_weights(4) = Assembly quality
	z_weights = [.35 .25 .15 .25];
	
	% Define a clipping threshold. Alignments that have overhang of more then
	% this threshold are not considered
	clipping_thrs = 10;
	
	% Specifify the z-score distance that edges towards a 'reference node' will
	% get
	ref_distance = -10;

	% Specify an 'assembly quality' score for the reference nodes
	ref_quality = 1E-5;
	
	% Pseudonode method
	PseudoMethod = 'tiling';
    
    % Maximum size of a pseudo node
    max_ref_node_size = inf;
    
    % Greedy extend option of the assembly path
    min_alen_extend   = 400;
    min_ident_extend  = 99;
    
    % Replace bases from pseudo nodes by the character 'N'
    ReplacePseudo = true;
    
    max_backtrack_depth = 1;
    
    ref_first = true;
    
    % Filter containment edges from the graph (i.e. contig completely
    % contained in another (depends on clipping thershold)
    containment_edge  = true;
    merge_overlapping = false;
    
% 3. Read optional parameters
   % Get VARARGIN user input
    if length(varargin) > 0
        while 1
            switch varargin{1}
                %case 'Startnode_name'
                %    Startnode_name = varargin{2};
                %case 'Endnode_name'
                %    Endnode_name = varargin{2};
                %case 'z_weigths'
                %    z_weights = varargin{2};
                %case 'ref_distance' 
                %    ref_distance = varargin{2};
                %case 'ref_quality' 
                %    ref_quality = varargin{2};
                %case 'max_chromosome_dist' 
                %    max_chromosome_dist = varargin{2};
                %case 'quit_treshold' 
                %    quit_treshold = varargin{2};
                %case 'tabu_time' 
                %    tabu_time = varargin{2};
                %case 'minimum_improvement' 
                %    minimum_improvement = varargin{2};
                %case 'AllowReverse'
                %    AllowReverse = varargin{2};
                case 'clipping_thrs' 
                    clipping_thrs = varargin{2};
                case 'Assemblies_for_refnodes'
                    Assemblies_for_refnodes = (varargin{2});
                case 'extend_ends'
                    endextend = varargin{2};
                case 'ReferenceAlignment'
                    ReferenceAlignment = varargin{2};
                case 'max_ref_node_size'
                    max_ref_node_size = varargin{2};
                case 'min_alen_extend'
                    min_alen_extend = varargin{2};
                case 'min_ident_extend'
                    min_ident_extend = varargin{2};
                case 'ReplacePseudo'
                    ReplacePseudo = varargin{2};
                case 'max_backtrack_depth'
                    max_backtrack_depth = varargin{2};
                case 'containment_edge'
                    containment_edge = varargin{2};
                case 'ref_first'
                    ref_first = varargin{2};
                case 'mapfilter'
                    mapfilter = varargin{2};
                case 'alignlen'
                    alignlen = varargin{2};
                case 'ident'
                    ident = varargin{2};
                case 'merge_overlapping'
                    merge_overlapping = true;
                otherwise
                    error(['Input ' varargin{2} ' is not known']);
            end
            if length(varargin) > 2
                varargin = varargin(3:end);
            else
                break;
            end
        end
    end
        
    
        
% 2. Run MAIA

    % Open file for statistics
    % TODO write MAIA statistics to file
    [status,result] = unix('rm maia_stats.txt');
    file_1 = fopen('maia_stats.txt','w');

    % Initialize some variables
    ReferenceAlignment = 'NotYetDoneByMaia';
    deltafiles_ref     = [];
    contig_ref         = [];
    overlap_ref        = [];
    name_hash_ref      = [];
    
    
    for r = 1:length(ref)
        CurrentRef = ref(r);
        filename = 'TempMaiaRef.fa';
        if isempty(dir(filename))
            fastawrite(filename,CurrentRef);
        else
            !rm TempMaiaRef.fa
            fastawrite(filename,CurrentRef);
        end
        
        
        
        [adj_direct contig_direct overlap names assembly_path weigth_direct ...
         deltafiles deltafiles_ref ReferenceAlignment ...
         contig_ref overlap_ref name_hash_ref] ...
            = assembly_driver(Assemblies,reference,ref(r).Header, ...
              deltafiles_ref,contig_ref, overlap_ref, name_hash_ref, ...
              'ref_node_assemblies',Assemblies_for_refnodes,...
              'max_chromosome_dist',PseudoMethod, ...
              'ReferenceAlignment', ReferenceAlignment, ...
              'containment_edge', containment_edge, ...
              'clipping_thrs', clipping_thrs,...
              'ref_first', ref_first, ...
              'mapfilter',mapfilter, ...
              'alignlen', alignlen, ...
              'ident' , ident);


    % 2.1 Visualize graph results (red line is highest scoring path)
        % Your overlap graph is now done, and a highest scoring path is found with a Tabu search
        % You can inspect your graph with for example Cytoscape by exporting it to .xgmml
        numlabs = 1:size(adj_direct,1);                % Give the nodes a numerical label
        numlabs = cellstr(num2str(numlabs'));          % in the order they appear in the adjency matrix
        [nodes, edges] = maia2cytoscape(adj_direct,names,contig_direct,numlabs,assembly_path(1),adj_direct,assembly_path);
        export_xgmml([CurrentRef.Header '_maia.xgmml'],['maia_chr' num2str(r) '.xgmml'],nodes,edges)


    % 2.2 Greedy extend highest scoring path on both sides (Not applicable in this small example)
        if endextend
            assembly_path = extend_path(adj_direct,contig_direct,overlap,assembly_path,[],'forward',min_alen_extend,min_ident_extend);
            assembly_path = extend_path(adj_direct,contig_direct,overlap,assembly_path,[],'backward',min_alen_extend,min_ident_extend);
        end

    % 2.3 Split out too large pseudo nodes 

        %max_ref_node_size = 250;  % maximum size of a reference node to have in your assembly
        %min_alen_extend          = 400;  % Min alignment length for extension
        %min_ident_extend         = 99;   % Min alignment identity for extension
        % Split the assembly at thos refnodes, backtrack to a split position and extend
        ap_splitted = split_and_extend(adj_direct, assembly_path, contig_direct,overlap, max_ref_node_size,min_alen_extend, min_ident_extend, max_backtrack_depth);


    % 2.4 Generate alignment matrix
        [ams ass] = assemble_paths(ap_splitted, contig_direct, Assemblies, deltafiles,deltafiles_ref, 'TempMaiaRef.fa', overlap, 'merge_overlapping',merge_overlapping);


    % 2.5 Call consensus  
        [s, f, sources] = call_consensuses(ams,ass, Assemblies);

        
    % 2.6 Replace the pseudo nodes by N
        if ReplacePseudo
            for si = 1:length(s)
                RefAssemNumber = find(ismember(ass{si},'reference'));
                if ~isempty(RefAssemNumber)
                    s{si}(sources{si} == RefAssemNumber) = 'N';
                end
            end
        end

    % 2.7 Write the assembly to disk
        pos = 0;
        clear assembly;
        assembly(size(s,1)).Header   = [];
        assembly(size(s,1)).Sequence = [];
        for si = 1:size(s,1)
            pos = pos+1;
            assembly(pos).Header   = [CurrentRef.Header '_maia_contig_' num2str(si)];
            assembly(pos).Sequence = s{si};
        end

        asfilename = 'maia_assembly.fa';
        if ~isempty(dir(asfilename)) && r == 1
            unix(['rm ' asfilename]);
            warning (['File ' asfilename ' already exists, deleting previous version']);
        end
        fastawrite(asfilename,assembly);
            
        unix(['rm ' filename]);
        
        
     % 2.8 Save some statistics
        
        if isa(ap_splitted,'double')
            ap_splitted = {ap_splitted};
        end
        
        % Iterate through contigs
        for si = 1:size(s,1)
            % used assemblies
            unique_assembly_nrs = unique(sources{si});

            
            % Print name of contig
            name = [CurrentRef.Header '_maia_contig_' num2str(si)];
            fprintf(file_1,'\nContig: %s \n', name);

            % Print total number of bases in contig
            fprintf(file_1,'Total number of bases: %d\n', length(s{si}));
            
            % Print usage
            fprintf(file_1,'Assembly usage:\n'); 
            for as = 1:length(unique_assembly_nrs)           
                
                unique_assembly_name = ass{si}{unique_assembly_nrs(as),:};
                % Print assembly name and percentage
                nr_of_contigs = sum(ismember({contig_direct(ap_splitted{si}).assembly},unique_assembly_name));
                
                
                fprintf(file_1,'Used %d nucleotides from %s using %d contigs\n', ...
                    sum(sources{si} == unique_assembly_nrs(as)), unique_assembly_name, nr_of_contigs);
            end
        end
        
     
    end
    
    fclose(file_1);
    
    function As = loadassemblies(filename)
        % Load list of assemblies from tab delimited file
        fid = fopen(filename);
        a = textscan(fid,'%s\t%s\t%d');
        fclose(fid);
        
        As = {};
        
        for i = 1:size(a{1},1)
            As{i,1} = a{1}{i};
            As{i,2} = a{2}{i};
            As{i,3} = a{3}(i);
        end
        
    end
end

