function [nodes, edges] = maia2cytoscape(A, names,contig_direct, labels, anchor,less_sparse_A,p)
% draw_assembly_graph Draws an overlap graph from a contig alignment adjecny matrix
% INPUT
% A:          Adjency matrix
% NAMES:      Contig names
% GIVENUMBER: TRUE / FALSE
%             True: nodes will be numbered per contig set (based on regexp)
%             false: nodes will get their contig name as label
% LABELS:     List of nodelabels. Numbers will be given if labels is empty
% ANCHOR:     Only nodes will be plotted that are connected to node 'ANCHOR'
%
% Note: 
% - Nodes that match no expression will be white
% - A maximum of seven colors is available
%
%
% Author: Jurgen Nijkamp
% Date: 17 november 2009

    pathnames = cell(1, length(p));
    for ip = 1:length(p)
        pathnames{ip} = contig_direct(p(ip)).name;
    end

    % If no node labels are given, just number them
    if nargin < 3
        givenumber = false;
        num = 1:size(A,1);
        labels = cellstr(num2str(num'));
    elseif isempty(labels)
        givenumber = true;
        labels = zeros(size(A,1),1);
    else
        givenumber = false;
    end

    
    if nargin > 3
        if ~isempty(anchor)
            if isempty(less_sparse_A)
              Aundirect = make_undirected(A);
              [A, ~, names, labels, contig_direct] = remove_unconnected(A, Aundirect,anchor, names, labels, contig_direct);
            else
              less_sparse_A = make_undirected(less_sparse_A);
              [A, ~, names, labels, contig_direct] = remove_unconnected(A, less_sparse_A,anchor, names, labels, contig_direct);
            end
        end
    end
    
    % Make standard color white ('w' = 119)
    nodeColors = char(119*ones(size(A,1),1));
    nodeColors = cellstr(nodeColors);
    
    AssemblyNames = unique(arrayfun(@(x) x.assembly, contig_direct, 'UniformOutput', false));
    color_list = {'b' 'g' 'm' 'c' 'y' 'r' 'b'};
    if length(AssemblyNames) > length(color_list)
        error('More groups specified then colors available');
    end
    
    %fprintf('*****Legend*****\n')
    %for c = 1:length(AssemblyNames)
    %    cl = getcolor(color_list{c});
    %    fprintf('%s \t\t %s\n',AssemblyNames{c},cl);
    %end
    
    % Loop through all  regular expressions given in varargin
    for v = 1:length(AssemblyNames)

        % Find all locations in names matching the regular expression
        index = arrayfun(@(x) isequal(x.assembly,AssemblyNames{v}), contig_direct);
        
        % Give the appropiate contigs a label
        if givenumber
            labels(index) = 1:sum(index);
        end
        
        % Give the contigs matching the regular expression a color
        nodeColors(index) = color_list(v);
        
    end

    if givenumber
        labels =  cellstr(num2str(labels));
    end
    
    % Plot graph
    %graphlayout('adjMatrix',A,'nodeLabels',labels,'nodeColors',nodeColors,'undirected',false)
    
    [nodes, edges] = export_cytoscape(A,labels,contig_direct,pathnames,names);
    
    
    function [nodes, edges] = export_cytoscape(adj,labels,contig_direct,pathnames,names)
        % Exports an xgmml file based on the specified nodes and edges
        %
        %  INPUT
        %     filename    Name of the file to output
        %     graphname   Name of the graph in cytoscape
        %     nodes       Struct array containing structs representing nodes
        %                 node.id = id number, referenced from edges
        %                 node.label = label, displayed in graph
        %                 node.att.x = attribute x, can be browsed through in the node attribute browser
        %                 node.graphics.[borderwidth,width,heigh,color,type,etc.] = various fields specifying the graphical representation
        %     edges       Struct array containg structs representing edges
        %                 edge.source = node id
        %                 edge.target = node id
        %                 edge.att.x = attribute x, can be browsed through in the edge attribute browser
        %                 edge.graphics.[borderwidth,color,sourceArrow,targetArrow,etc.] = various fields specifiying the graphical representation
        %
        %    NOTE
        %     For possible attributes/values for the graphics fields, see JavaDoc
        %     documentation Cytoscape. 
        
        colors = {'#0000FF','#FF0000','#00FF00','#000000','#FFFF00','#00FFFF','#FF00FF'};
        
        assemblies = unique({contig_direct.assembly});
        
        nodes = struct;
        
        for i = 1:length(contig_direct)
           nodes(i).id = find(ismember(names,contig_direct(i).name));
           nodes(i).label = contig_direct(i).name;
           nodes(i).att.assembly_name = contig_direct(i).assembly;
           nodes(i).att.assembly_quality = contig_direct(i).assembly_quality;
           nodes(i).att.orient = contig_direct(i).orient;
           nodes(i).att.size = contig_direct(i).size;
           nodes(i).graphics.color = colors{ismember(assemblies,contig_direct(i).assembly)};
        end;
        
        edges = struct;
        
        
        [x,y] = find(adj);
        for i = 1:length(x)
           edges(i).source   = x(i);
           edges(i).target   = y(i);
           edges(i).label    = num2str(i);
           edges(i).graphics.color = '#000000';
           
           locx = find(ismember(pathnames,names(x(i))));
           locy = find(ismember(pathnames,names(y(i))));
           if ~isempty(locx) && ~isempty(locy)
               if locx == locy-1 || locx == locy+1
                   %logical((loc+1) < length(pathnames)) && logical(isequal(pathnames(loc+1),names(y(i))))
                    edges(i).graphics.color = '#FF3300';
               end
           end
        end      
        
    end      
  
    function ColorWord = getcolor(ColorLetter)
        switch ColorLetter
            case 'b'
                ColorWord = 'blue';
            case 'g'
                ColorWord = 'green';                
            case 'm'
                ColorWord = 'magenta';    
            case 'c'
                ColorWord = 'cyan';
            case 'y'
                ColorWord = 'yellow';
            case 'r'
                ColorWord = 'red';
        end
    end

    


    function [A, Aundirect, names, labels, contig_direct] = remove_unconnected(A, Aundirect, u, names, labels, contig_direct)
        dist = dfs(Aundirect,u);
        unconnected = dist == -1;
        A(unconnected,:) = [];
        A(:,unconnected) = [];
        if ~isempty(names)
            names(unconnected) = [];
        end
        if ~isempty(labels)
            labels(unconnected) = [];
        end
        if ~isempty(contig_direct)
            contig_direct(unconnected) = [];
        end
        
    end

end