 function [deltafile Rseq Qseq] = normalize_delta(deltafile,Rname,Rseq,Rorient,Qname,Qseq, Qorient)
    % Transforms pairwise alignments in the form of:
    %    
    %    5' ------ 3'           deltafile.R
    %           || 
    %        5' ------ 3'       deltafile.Q
    %
    %
    % Indels:
    % Positive = insert in Current
    % Negative = insert in Previous
    %
    %
    % INPUT
    % deltafile     Deltafile with Q and R alignment
    % Rname         Name of contig to be R in output
    % Rseq          Sequence of contig
    % Rorient       Orientation of the sequence
    % Qname         Name of contig to be Q in output
    % Qseq          Sequence of contig
    % Qorient       Orientation of the sequence
    %
    % OUTPUT
    % deltafile     Deltafile qith desired contig on R position and in 5-3 orientation
    % Rseq          Sequence od R contig in 5-3 orientation
    % Qseq          Sequence of Q contig in 5-3 orientation
    
    
    
%     There are eight possible (relevant) situations:
%     1. The contig you want on position R is already on position R 
%         1a) Rseq in 5'- 3' and Qseq in 5'- 3' orientation
%         1b) Rseq in 5'- 3' and Qseq in 3'- 5' orientation
%         1c) Rseq in 3'- 5' and Qseq in 5'- 3' orientation
%         1d) Rseq in 3'- 5' and Qseq in 3'- 5' orientation
%     2. The contig you want on position R is on position Q
%         2a) Rseq in 5'- 3' and Qseq in 5'- 3' orientation
%         2b) Rseq in 5'- 3' and Qseq in 3'- 5' orientation
%         2c) Rseq in 3'- 5' and Qseq in 5'- 3' orientation
%         2d) Rseq in 3'- 5' and Qseq in 3'- 5' orientation

% In situation 2 the R and Q contig have to be switched. After the switch
% there are only four situations left, namely 1a-d. 
% 
% ------------------------------------------------------------------------
% Actions to be taken:
% 1a)
% nothing to be done
% 
% 1b)
% take reverse complement of Qseq and adapt Q alignment positions
% 
% 1c)
% take reverse complement of Rseq and adatp R alignment positions
% reverse Q alignment positions
% reversel indels, because alignment is reversed
% 
% 1d)
% take reversecomplement of Rseq and Qseq and adapt alignment positions
% reversel indels, because alignment is reversed
% 
% 2a)
% Switch R and Q 
% 
% 
% 2b)
% Switch R and Q
% take reverse complement of Qseq and adapt alignment positions
% reverse Rseq alignment positions
% reversel indels, because alignment is reversed
% 
% 2c)
% Switch R and Q
% take reverse complement of Rseq and adapt alignment positions
% 
% 2d)
% Switch R and Q
% take reversecomplement of Rseq and Qseq and adapt alignment positions
% reversel indels, because alignment is reversed
%
% ------------------------------------------------------------------------


    % Get one of the above described situations
    situation = get_situation(deltafile, Rname, Qname, Rorient,Qorient);


    switch situation
        case '1a'
            return;

        case '1b'
            [deltafile Qseq] = get_reverse_complement(deltafile, Qname, Qseq);

        case '1c'
            [deltafile Rseq] = get_reverse_complement(deltafile, Rname, Rseq);
            deltafile        = reverse_alignment(deltafile, Qname);
            deltafile        = reverse_alignment(deltafile, Rname);
            deltafile        = reverse_indels(deltafile);

        case '1d'
            [deltafile Rseq] = get_reverse_complement(deltafile, Rname, Rseq);
            [deltafile Qseq] = get_reverse_complement(deltafile, Qname, Qseq);
            deltafile        = reverse_alignment(deltafile, Qname);
            deltafile        = reverse_alignment(deltafile, Rname);
            deltafile        = reverse_indels(deltafile);

        case '2a'
            deltafile = switchRQ(deltafile);

        case '2b'
            deltafile = switchRQ(deltafile);
            [deltafile Qseq] = get_reverse_complement(deltafile, Qname, Qseq);
            deltafile        = reverse_alignment(deltafile, Rname);
            deltafile        = reverse_alignment(deltafile, Qname);
            deltafile        = reverse_indels(deltafile);

        case '2c'
            deltafile = switchRQ(deltafile);
            [deltafile Rseq] = get_reverse_complement(deltafile, Rname, Rseq);

        case '2d'
            deltafile = switchRQ(deltafile);
            [deltafile Rseq] = get_reverse_complement(deltafile, Rname, Rseq);
            [deltafile Qseq] = get_reverse_complement(deltafile, Qname, Qseq);
            deltafile        = reverse_alignment(deltafile, Qname);
            deltafile        = reverse_alignment(deltafile, Rname);
            deltafile        = reverse_indels(deltafile);


        otherwise
            error('No situation found');
    end



    function sit = get_situation(deltafile, Rname, Qname, Rorient, Qorient)

     % Situation 1
     if isequal(deltafile.R, Rname) && isequal(deltafile.Q, Qname)
         % Orientation situation
         if Rorient == 53 && Qorient == 53
             sit = '1a';
         elseif Rorient == 53 && Qorient == 35
             sit = '1b';
         elseif Rorient == 35 && Qorient == 53
             sit = '1c';
         elseif Rorient == 35 && Qorient == 35
             sit = '1d';
         else
             error('could not determine orientation');
         end

     % Situation 2    
     elseif isequal(deltafile.Q, Rname) && isequal(deltafile.R, Qname)
         % Orientation situation
         if Rorient == 53 && Qorient == 53
             sit = '2a';
         elseif Rorient == 53 && Qorient == 35
             sit = '2b';
         elseif Rorient == 35 && Qorient == 53
             sit = '2c';
         elseif Rorient == 35 && Qorient == 35
             sit = '2d';
         else
             error('could not determine orientation');
         end

     else
         error('This is not the correct deltafile');

     end
    end

    function d = switchRQ(d)
    % Switch position R and Q
    
        % Save old values of R
        Stemp    = d.S1;
        Etemp    = d.E1;
        Ltemp    = d.LENR;
        Rtemp    = d.R;
        
        % Put new values in R
        d.S1     = d.S2;
        d.E1     = d.E2;
        d.LENR   = d.LENQ;
        d.R      = d.Q;
        
        % Put new values in Q
        d.S2     = Stemp;
        d.E2     = Etemp;
        d.LENQ   = Ltemp;
        d.Q      = Rtemp;
        
        % Inverse sign of indel
        d.indels = -d.indels;
    end
 
    function [d seqr] = get_reverse_complement(d, name, seq)
        % Get the start ans stop position in reverse complement format

        %       S           E
        % 3' -------------------- 5'
        
        %           ||       
        %           \/
        %       Er          Sr
        % 5' -------------------- 3'
        
        if isequal(d.R, name)
            d.S1 = length(seq) - d.S1 + 1;
            d.E1 = length(seq) - d.E1 + 1;
            
        elseif isequal(d.Q, name)
            d.S2 = length(seq) - d.S2 + 1;
            d.E2 = length(seq) - d.E2 + 1;
            
        else
            error('Contig name not found in this deltafile')
        end
                    
        seqr = seqrcomplement(seq);
        
    end

    function d = reverse_alignment(d, name)
    % Reverse alignment of 'name'

        if isequal(d.R, name)        
            % Switch the coordinates of an alignment
            temp = d.S1;
            d.S1 = d.E1;
            d.E1 = temp;            
        elseif isequal(d.Q, name)   
            temp = d.S2;
            d.S2 = d.E2;
            d.E2 = temp;  
        else
            error('Contig name not found in this deltafile')
        end            


    end

    function d = reverse_indels(d)
        % If the alignment is reversed the indels has to be reversed as well
        
        if isempty(d.indels)
            return
        end
        
        % Alignment length = endpos- startpos + number of insertions
        A_length_R = abs(d.E1 - d.S1) + 1 + sum(d.indels < 0);
        A_length_Q = abs(d.E2 - d.S2) + 1 + sum(d.indels > 0);
        
        if A_length_R == A_length_Q
            A_length = A_length_R;
        else
            error('Alignemt lengths are not the same');
        end
                
        % First indel offset is Alignment length - som(all offsets) + 1 
        indel_offset = A_length - sum(abs(d.indels)) + 1;
        
        new_indels   = zeros(size(d.indels));
        
        % Set first indel to indels offset
        if d.indels(end) > 0
            new_indels(1) = indel_offset;
        else
            new_indels(1) = -indel_offset;
        end
        
        % Reverse indel list
        new_indels_pos = 2;
        for indi = length(d.indels):-1:2
            if d.indels(indi-1) > 0
                new_indels(new_indels_pos) = abs(d.indels(indi));
            else
                new_indels(new_indels_pos) = -abs(d.indels(indi));
            end
            new_indels_pos = new_indels_pos + 1;
        end
                
        d.indels = new_indels;
    end


end